/*----------------------------------------------------------------------
 *
 * Dump AES header
 * Assumes file given has been encrypted using mqput v2 or above.
 *
 * History
 * ----------------------------------------------------------------------
 * 02-02-2007  sidy  Began code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <malloc.h>

#include "rijndael.h"


/*
 * header block used for AES/Rijndael encrypted messages
 *
 */
struct HDR_BLOCK
{
	unsigned char hdr_length[4];
	unsigned char hdr_major;
	unsigned char hdr_minor;
	char src_hash[34];
	unsigned char file_length[4];
	char file_name[64];
	char file_type[4];
	unsigned char hdr_flag_1;	/* flag used to hold 'E'ncrypted data block */
	unsigned char hdr_flag_2;
	unsigned char hdr_flag_3;
	unsigned char hdr_flag_4;	/* OS Flag for PUT utility */
};







/**
 * getHeaderLength()
 *
 *
 * Utility to return length of header
 * given pointer to header block
 *
 *
 */
long getHeaderLength( struct HDR_BLOCK *hdr )
{
static long _HdrBlockLength;

	_HdrBlockLength =hdr->hdr_length[0];
	_HdrBlockLength|=hdr->hdr_length[1]<<8;
	_HdrBlockLength|=hdr->hdr_length[2]<<16;
	_HdrBlockLength|=hdr->hdr_length[3]<<24;
	return _HdrBlockLength;
}








/**
 * getFileLength()
 *
 *
 * Helper utility to return file length stored in 
 * header block passed in message buffer.
 *
 *
 *
 *
 */
long getFileLength( struct HDR_BLOCK *hdr )
{
static long _SavedFileLength;

	_SavedFileLength= hdr->file_length[0];
	_SavedFileLength|=hdr->file_length[1]<<8;
	_SavedFileLength|=hdr->file_length[2]<<16;
	_SavedFileLength|=hdr->file_length[3]<<24;
	return _SavedFileLength;
}




struct stat fileStat;
char *pSrcBuffer;	/* clear text data from file */
char *ptrDestBuffer;
char *pDestBuffer;	/* destination pointer of encryption buffer */
char *endPtr;
char *workPtr;
FILE *fPtr;
long fileSize;
char key[32];
char szFileName[64];
char szAESKey[64];
char szHash[64];


/**
 *
 * AllocateAndReadFile()
 *
 * Given a file name, attempt to size it and then allocate space if it
 * is a valid file.
 *
 * @return   int 0= OK
 */
int AllocateAndReadFile(char *fileName )
{
char thisChar;

	if(!stat( fileName, &fileStat))
		fileSize = fileStat.st_size;
	else
	{
		printf("ERROR - Cannot STAT file, check spelling!\n\n");
		exit(1);
	}
	if(S_ISDIR(fileStat.st_mode))
	{
		printf("ERROR - Cannot process a directory!\n\n");
		exit(3);
	}
/*
 *  Attempt to allocate memory and open file
 */
	if((pSrcBuffer = (char*) malloc( fileSize+1)) != NULL)
		workPtr = pSrcBuffer;
	else
	{
		printf("ERROR - Cannot Allocate any memeory!\n\n");
		exit(4);
	}
	if((fPtr=fopen(fileName,"r"))!=NULL)
	{
		while(!feof(fPtr))
		{
			thisChar = fgetc(fPtr);
			if(feof(fPtr)) break;
			*workPtr++ = thisChar;
		}
		endPtr = workPtr-1;
		fclose(fPtr);
	}
	else
	{
		printf("ERROR - Cannot open file for reading!\n\n");
		free(pSrcBuffer);
		exit(5);
	}
	return 0;
}



/**
 * DumpBlock()
 *
 *
 * Dump the Header to screen if the -dh flag
 * is specified on the command line
 *
 * @pre    None
 * @post   None
 * @return void
 */
void DumpBlock(struct HDR_BLOCK *hdr)
{
	printf("Header Dump\n");
	printf("===========\n");
	printf("Version    [%c.%c] \n", hdr->hdr_major, hdr->hdr_minor);
	printf("Block Size [%ld]\n", getHeaderLength(hdr));
	printf("Hash       [%s]\n", hdr->src_hash);
	printf("File Name  [%s]\n", hdr->file_name);
	printf("File Length[%ld]\n",getFileLength(hdr));
	printf("FileType   [%s] MQLink file type rule\n", hdr->file_type);
	printf("Flag 1     [%c] Encrypt Flag\n", hdr->hdr_flag_1);
	printf("Flag 2     [%c] Spare\n", hdr->hdr_flag_2);
	printf("Flag 3     [%c] Spare\n", hdr->hdr_flag_3);
	printf("Flag 4     [%c] OS Type\n", hdr->hdr_flag_4);
}





/*=================================================================
 *
 *
 *
 *
 *                      M A I N
 *
 *
 *
 *
 *
 */
int main(int argc, char **argv)
{
int i;


struct HDR_BLOCK *hdr;
rijndael_ctx *ctx;


	printf("\nDumpHdr v1.0 - Dump AES encrypted file details\n\n");
	printf("Build Date: %s  Time: %s\n", __DATE__, __TIME__);
	printf("Author: Sid Young\n");
	if (argc < 2)
	{
		printf("Required parameter missing:\n\n");
		printf("\t%s <file_to_dump>\n\n\n", argv[0]);
		exit(99);
	}
/*
 *---------------------------------------------
 *       Process command line args
 *---------------------------------------------
 */
	if(argc>1)
	{
		printf("Command line options:\n");
		for(i=0;i<argc;++i)
		{
 			printf("Arg [%d] Value [%s]\n",i, argv[i]); 
			if(!strcmp(argv[i],"-?"))
			{
				printf("HELP: %s <file> [-key <key>] [-dh]\n",argv[0]);
				printf(" <file> - Name of file to send\n");
				printf(" -key <key> - 16 byte hex string for cipher\n");
				exit(0);
			}
			if(!strcmp(argv[i],"-key")) {strcpy(szAESKey,argv[i+1]);}
		}
	}
	strcpy(szFileName, argv[1]);
/*
 *---------------------------------------------
 * Allocate a header to hold details about message
 *---------------------------------------------
 */
  	if((hdr=(struct HDR_BLOCK *)malloc(sizeof(struct HDR_BLOCK)))==NULL)
	{
		printf("ERROR - Cannot allocate HDR block\n");
		exit(-1);
	}
	memset(hdr,0,sizeof(struct HDR_BLOCK));
/*
 *---------------------------------------------
 * Allocate a context buffer for crypto call.
 *---------------------------------------------
 */
	if((ctx=(rijndael_ctx *)malloc(sizeof(rijndael_ctx)))==NULL)
	{
		printf("ERROR - Cannot allocate CTX block\n");
		exit(-1);
	}
/*
 *---------------------------------------------
 * make sure key is correct length 128 bits minimum
 *---------------------------------------------
 */
	while(strlen(szAESKey)<32)
		strcat(szAESKey, "0");
	memcpy(key,szAESKey,32);
	printf("AES Key [%s]",key);
/*
 *---------------------------------------------
 *  Read from the file and put the message buffer to the message queue
 *  Loop until null line or end of file, or there is a failure
 *---------------------------------------------
 */
	AllocateAndReadFile( szFileName );
	memcpy(hdr, pSrcBuffer, sizeof(struct HDR_BLOCK));
	DumpBlock( hdr );
	free(pSrcBuffer);
	pSrcBuffer=NULL;
	return(0);
}
/*
 * end of file
 */
